<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * 権限登録時バリデート
 *
 * @package     NetCommons.validator
 * @author      Noriko Arai,Ryuji Masukawa
 * @copyright   2006-2007 NetCommons Project
 * @license     http://www.netcommons.org/license.txt  NetCommons License
 * @project     NetCommons Project, supported by National Institute of Informatics
 * @access      public
 */
class Authority_Validator_Regist extends Validator
{
	//プライベート変数
	var $_configs = null;

	/**
	 * validate実行
	 *
	 * @param   mixed   $attributes チェックする値
	 * @param   string  $errStr     エラー文字列
	 * @param   array   $params     オプション引数
	 * @return  string  エラー文字列(エラーの場合)
	 * @access  public
	 */
	function validate($attributes, $errStr, $params)
	{
		//パラメータ取得
		$module_id = intval($attributes["module_id"]);
		$role_authority_id = intval($attributes["role_authority_id"]);
		$role_authority_name = $attributes["role_authority_name"];
		$myroom_use_flag = intval($attributes["myroom_use_flag"]);
		$allow_attachment = $this->_getAllowAttachment(intval($attributes["allow_attachment"]));
		$allow_htmltag_flag = intval($attributes["allow_htmltag_flag"]);
		$allow_layout_flag = intval($attributes["allow_layout_flag"]);
		
		$enroll_sysmodules_id = $attributes["enroll_sysmodules_id"];
		$enroll_modules_id = $attributes["enroll_modules_id"];
		
		$usermodule_auth = $attributes["usermodule_auth"];
		
		$public_createroom_flag = intval($attributes["public_createroom_flag"]);
		$group_createroom_flag = intval($attributes["group_createroom_flag"]);
		
		$user_authority_id = intval($attributes["user_authority_id"]);
		
		// container取得
		$container =& DIContainerFactory::getContainer();
		
		//使用コンポーネント取得
		$configView =& $container->getComponent("configView");
		$authoritiesView =& $container->getComponent("authoritiesView");
		$actionChain =& $container->getComponent("ActionChain");
		
		// configデータ取得
		$this->_configs =& $configView->getConfigByCatid($module_id, _MAIN_CONF_CATID, null, null, array($this, "_fetchcallbackConfig"));
		if($this->_configs === false) {
			return sprintf(_INVALID_SELECTDB,"config");	
		}
		
		$configs =& $configView->getConfigByCatid($module_id, AUTHORITY_CONF_CATID_MAX_SIZE);
		if($configs === false) {
			return sprintf(_INVALID_SELECTDB,"config");	
		}
		
		foreach ($configs as $config) {
			$this->_configs["conf_max_sizes"][] = $config["conf_value"];
		}
		$this->_configs["conf_max_size_count"] = count($this->_configs["conf_max_sizes"]);
		if ($myroom_use_flag == _ON) {
			$max_size = $this->_getMaxSize(intval($attributes["max_size"]));
		} else {
			$max_size = 0;
		}
		
		if ($role_authority_id > 0) {
			//既存データ(権限編集)
			$authority =& $authoritiesView->getAuthorityById($role_authority_id);
			if ($authority === false || !isset($authority['role_authority_id'])) {
				return $errStr;	
			}
			
			$system_flag = $authority["system_flag"];
			if ($authority["system_flag"] == _ON && $authority["user_authority_id"] == _AUTH_ADMIN) {
				//サイト管理者
				$system_flag = _ON;
			} else {
				//それ以外
				//---HTMLの許可---
				if ($this->_configs["htmltag_disabled"] == _ON) {
					$allow_htmltag_flag = _OFF;
				}
				//---ベース権限---
				if ($authority["system_flag"] == _ON) {
					$user_authority_id = $authority["user_authority_id"];
				}
				$system_flag = $authority["system_flag"];
			}
		} else {
			//新規データ(権限追加)
			//---HTMLの許可---
			if ($this->_configs["htmltag_disabled"] == _ON) {
				$allow_htmltag_flag = _OFF;
			}
			//---システムフラグ---
			$system_flag = _OFF;
		}
		
		if ($this->_configs["htmltag_disabled"] != _ON) {
			$allow_htmltag_flag = $this->_getAllowHtmlTag($user_authority_id);
		} else {
			$allow_htmltag_flag = $allow_htmltag_flag;
		}
			
		//バリデートも行ったため、Actionにセット
		$action =& $actionChain->getCurAction();
		
		$attributes = array(
			"module_id" => $module_id,
			"role_authority_id" => $role_authority_id,
			"role_authority_name" => $role_authority_name,
			"myroom_use_flag" => $myroom_use_flag,
			"allow_attachment" => $allow_attachment,
			"allow_htmltag_flag" => $allow_htmltag_flag,
			"allow_layout_flag" => $allow_layout_flag,
			"enroll_sysmodules_id" => $enroll_sysmodules_id,
			"enroll_modules_id" => $enroll_modules_id,
			"usermodule_auth" => $usermodule_auth,
			"public_createroom_flag" => $public_createroom_flag,
			"group_createroom_flag" => $group_createroom_flag,
			"max_size" => $max_size,
			"user_authority_id" => $user_authority_id,
			"system_flag" => $system_flag
		);
		
		BeanUtils::setAttributes($action, $attributes);
	}
	
	/**
	 * fetch時コールバックメソッド
	 * @result adodb object
	 * @return array configs
	 * @access	private
	 */
	function _fetchcallbackConfig($result) 
	{
		$config = array();
		while ($row = $result->fetchRow()) {
			$config[$row["conf_name"]] = $row["conf_value"];
		}
		return $config;
	}

	function _getMaxSize($val)
	{
		for ($i=0; $i<$this->_configs["conf_max_size_count"]; $i++) {
			if (isset($this->_configs["conf_max_sizes"][$i+1])) {
				if ($i == 0 && $val < $this->_configs["conf_max_sizes"][$i]) {
					return $val;
				} elseif ($this->_configs["conf_max_sizes"][$i] <= $val && $this->_configs["conf_max_sizes"][$i+1] > $val) { 
					return $this->_configs["conf_max_sizes"][$i];
				}
			} else {
				return $this->_configs["conf_max_sizes"][$i];
			}
		}
		return 0;
	}
	function _getAllowAttachment($val)
	{
		switch ($val) {
			case _ALLOW_ATTACHMENT_ALL:
				return intval(_ALLOW_ATTACHMENT_ALL);
			case _ALLOW_ATTACHMENT_IMAGE:
				return intval(_ALLOW_ATTACHMENT_IMAGE);
			default:
				return intval(_ALLOW_ATTACHMENT_NO);
		}

	}
	function _getAllowHtmlTag($user_authority_id)
	{
		$key = "";
		switch ($user_authority_id) {
		case _AUTH_ADMIN:
			$key = "_admin";
			break;
		case _AUTH_CHIEF:
			$key = "_chief";
			break;
		case _AUTH_MODERATE:
			$key = "_moderat";
			break;
		case _AUTH_GENERAL:
			$key = "_general";
			break;
		case _AUTH_GUEST:
			$key = "_guest";
			break;
		default:
		}
		if (isset($this->_configs["htmltag_auth".$key])) {
			return _ON;
		} else {
			return _OFF;
		}
	}

}
?>
