<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * アップデートクラス
 *
 * @package     NetCommons
 * @author      Noriko Arai,Ryuji Masukawa
 * @copyright   2006-2007 NetCommons Project
 * @license     http://www.netcommons.org/license.txt  NetCommons License
 * @project     NetCommons Project, supported by National Institute of Informatics
 * @access      public
 */
class Whatsnew_Update extends Action
{
	//使用コンポーネントを受け取るため
	var $db = null;

    /**
     * execute実行
     *
     * @access  public
     */
	function execute()
	{
		$adodb = $this->db->getAdoDbObject();
		$metaTables = $adodb->MetaTables();
		if (!in_array($this->db->getPrefix()."whatsnew_user", $metaTables)) {
			$sql = "CREATE TABLE `".$this->db->getPrefix()."whatsnew_user` (" .
					" `whatsnew_id` int(11) NOT NULL default 0," .
					" `user_id` varchar(40) NOT NULL," .
					" `room_id` int(11) default NULL," .
					" PRIMARY KEY (`user_id`, `whatsnew_id`)," .
					" KEY `whatsnew_id` (`whatsnew_id`)," .
					" KEY `user_id` (`user_id`)" .
					") TYPE=MyISAM;";
			$result = $this->db->execute($sql);
			if ($result === false) {
				return false;
			}
		}
		if (!in_array($this->db->getPrefix()."whatsnew_select_room", $metaTables)) {
			$sql = "CREATE TABLE `".$this->db->getPrefix()."whatsnew_select_room` (" .
					" `block_id` int(11) unsigned NOT NULL," .
					" `room_id` int(11) unsigned NOT NULL," .
					" PRIMARY KEY (`block_id`, `room_id`)" .
					") TYPE=MyISAM;";
			$result = $this->db->execute($sql);
			if ($result === false) {
				return false;
			}
		}

		$metaColumns = $adodb->MetaColumns($this->db->getPrefix()."whatsnew_user");
		if (!isset($metaColumns["room_id"]) && !isset($metaColumns["ROOM_ID"])) {
			$sql = "ALTER TABLE `".$this->db->getPrefix()."whatsnew_user"."` ADD `room_id` INT(11) NULL DEFAULT NULL AFTER `user_id` ;";
			$result = $this->db->execute($sql);
			if ($result === false) {
				return false;
			}
		}

		$metaColumns = $adodb->MetaColumns($this->db->getPrefix()."whatsnew_block");
		if (!isset($metaColumns["select_room"]) && !isset($metaColumns["SELECT_ROOM"])) {
			$sql = "ALTER TABLE `".$this->db->getPrefix()."whatsnew_block"."` CHANGE `current_room` `select_room` TINYINT( 1 ) UNSIGNED NOT NULL DEFAULT 0 ;";
			$result = $this->db->execute($sql);
			if ($result === false) {
				return false;
			}
		}

		$metaColumns = $adodb->MetaColumns($this->db->getPrefix()."whatsnew_block");
		if (!isset($metaColumns["myroom_flag"]) && !isset($metaColumns["MYROOM_FLAG"])) {
			$sql = "ALTER TABLE `".$this->db->getPrefix()."whatsnew_block"."` ADD `myroom_flag` TINYINT(1) NULL DEFAULT 0 AFTER `select_room` ;";
			$result = $this->db->execute($sql);
			if ($result === false) {
				return false;
			}
		}

		$metaColumns = $adodb->MetaColumns($this->db->getPrefix()."whatsnew");
		if (isset($metaColumns["UNIQUE_ID"]) && $metaColumns["UNIQUE_ID"]->type != "varchar") {
			$sql = "ALTER TABLE `".$this->db->getPrefix()."whatsnew"."` CHANGE `unique_id` `unique_id` VARCHAR(40) NOT NULL DEFAULT '0' ;";
			$result = $this->db->execute($sql);
			if ($result === false) {
				return false;
			}
		}

		$metaColumns = $adodb->MetaColumns($this->db->getPrefix()."whatsnew");
		if (!isset($metaColumns["count_num"]) && !isset($metaColumns["COUNT_NUM"])) {
			$sql = "ALTER TABLE `".$this->db->getPrefix()."whatsnew"."` ADD `count_num` INT(11) UNSIGNED NOT NULL DEFAULT 0 AFTER `parameters` ;";
			$result = $this->db->execute($sql);
			if ($result === false) {
				return false;
			}
		}

		$metaColumns = $adodb->MetaColumns($this->db->getPrefix()."whatsnew");
		if (!isset($metaColumns["child_flag"]) && !isset($metaColumns["CHILD_FLAG"])) {
			$sql = "ALTER TABLE `".$this->db->getPrefix()."whatsnew"."` ADD `child_flag` INT(11) UNSIGNED NOT NULL DEFAULT 0 AFTER `count_num` ;";
			$result = $this->db->execute($sql);
			if ($result === false) {
				return false;
			}
			// count_numが０以上のものがあれば、child_flag=1のデータとして新着テーブルにinsert
			// 親のcount_numは０に戻す
			$where_params = array(
				"count_num!=0" => null
			);
			$whatsnews = $this->db->selectExecute("whatsnew", $where_params);
			foreach($whatsnews as $whatsnew) {
				$params = $whatsnew;
				$whatsnew_id = $params['whatsnew_id'];
				unset($params['whatsnew_id']);
				$params['child_flag'] = _ON;
				$params['description'] = "";
				$result = $this->db->insertExecute("whatsnew", $params, false, "whatsnew_id");
				// 日誌のみ新着の日付振り替え処理を実装
				$post_where_params = array(
					"post_id" => $params['unique_id']
				);
				$journal_post = $this->db->selectExecute("journal_post", $post_where_params, null, 1);
				if(isset($journal_post[0]) && $journal_post[0]['journal_date'] != "") {
					$params['insert_time'] = $journal_post[0]['journal_date'];
					$params['update_time'] = $journal_post[0]['journal_date'];
				}
				
				$result = $this->db->updateExecute("whatsnew", array("count_num"=> 0, "insert_time" => $params['insert_time'], "update_time" => $params['update_time']), array("whatsnew_id"=>$whatsnew_id));
			}
		}

		return true;
	}
}
?>
